@@ -136,20 +136,22 @@ zfsdev_state_init(struct file *filp)
 	return (0);
 }
 
-static void
+static int
 zfsdev_state_destroy(struct file *filp)
 {
-	zfsdev_state_t *zs = filp->private_data;
+	zfsdev_state_t *zs;
 
-	ASSERT(zs != NULL);
-	ASSERT3S(zs->zs_minor, >, 0);
+	ASSERT(MUTEX_HELD(&zfsdev_state_lock));
+	ASSERT(filp->private_data != NULL);
 
+	zs = filp->private_data;
+	zs->zs_minor = -1;
 	zfs_onexit_destroy(zs->zs_onexit);
 	zfs_zevent_destroy(zs->zs_zevent);
 	zs->zs_onexit = NULL;
 	zs->zs_zevent = NULL;
-	membar_producer();
-	zs->zs_minor = -1;
+
+	return (0);
 }
 
 static int
@@ -167,9 +169,13 @@ zfsdev_open(struct inode *ino, struct file *filp)
 static int
 zfsdev_release(struct inode *ino, struct file *filp)
 {
-	zfsdev_state_destroy(filp);
+	int error;
 
-	return (0);
+	mutex_enter(&zfsdev_state_lock);
+	error = zfsdev_state_destroy(filp);
+	mutex_exit(&zfsdev_state_lock);
+
+	return (-error);
 }
 
 static long
@@ -206,6 +212,12 @@ zfs_max_nvlist_src_size_os(void)
 	return (MIN(ptob(zfs_totalram_pages) / 4, 128 * 1024 * 1024));
 }
 
+/* Update the VFS's cache of mountpoint properties */
+void
+zfs_ioctl_update_mount_cache(const char *dsname)
+{
+}
+
 void
 zfs_ioctl_init_os(void)
 {
@@ -276,6 +288,8 @@ zfsdev_detach(void)
 #define	ZFS_DEBUG_STR	""
 #endif
 
+zidmap_t *zfs_init_idmap;
+
 static int __init
 openzfs_init(void)
 {
@@ -299,6 +313,8 @@ openzfs_init(void)
 	printk(KERN_NOTICE "ZFS: Posix ACLs disabled by kernel\n");
 #endif /* CONFIG_FS_POSIX_ACL */
 
+	zfs_init_idmap = (zidmap_t *)zfs_get_init_idmap();
+
 	return (0);
 }
 
